import {HTTP_INTERCEPTORS, HttpClient, HttpClientModule} from "@angular/common/http";
import {APP_INITIALIZER, ErrorHandler, NgModule} from "@angular/core";
import {MatPaginatorIntl, MatSnackBar} from "@angular/material";
import {BrowserModule} from "@angular/platform-browser";
import {BrowserAnimationsModule} from "@angular/platform-browser/animations";
import {AppRoutingModule} from "@app/app-routing.module";
import {MainToolbarMobilePresentational} from "@app/components/presentationals/main-toolbar/main-toolbar-mobile/main-toolbar-mobile.presentational";
import {ThemeSelectorPresentational} from "@app/components/presentationals/theme-selector/theme-selector.presentational";
import {ApiModule} from "@app/generated-api/api.module";
import {SharedModule} from "@app/shared/shared.module";
import {AppState} from "@app/stores/app.state";
import {TranslateLoader, TranslateModule} from "@ngx-translate/core";
import {TranslateHttpLoader} from "@ngx-translate/http-loader";
import {NgxsReduxDevtoolsPluginModule} from "@ngxs/devtools-plugin";
import {NgxsLoggerPluginModule} from "@ngxs/logger-plugin";
import {NgxsRouterPluginModule} from "@ngxs/router-plugin";
import {NgxsModule} from "@ngxs/store";
import {AppConfigService, ENVIRONMENT, GlobalErrorsHandlerService, OAuth2Interceptor, OAuthStorage, SNACK_BAR} from "solidify-frontend";
import {environment} from "../environments/environment";
import {AppComponent} from "./app.component";
import {FooterPresentational} from "./components/presentationals/footer/footer.presentational";
import {HomePresentational} from "./components/presentationals/home/home.presentational";
import {LanguageSelectorPresentational} from "./components/presentationals/language-selector/language-selector.presentational";
import {LoginPresentational} from "./components/presentationals/login/login.presentational";
import {MainToolbarDesktopPresentational} from "./components/presentationals/main-toolbar/main-toolbar-desktop/main-toolbar-desktop.presentational";
import {PageNotFoundPresentational} from "./components/presentationals/page-not-found/page-not-found.presentational";
import {CustomMatPaginatorIntlService} from "./shared/services/custom-mat-paginator-intl.service";

const components = [
  AppComponent,
  FooterPresentational,
  HomePresentational,
  LanguageSelectorPresentational,
  LoginPresentational,
  MainToolbarDesktopPresentational,
  MainToolbarMobilePresentational,
  PageNotFoundPresentational,
  ThemeSelectorPresentational,
];

const appInitializerFn = (appConfig: AppConfigService) => () => appConfig.mergeConfig(environment).toPromise();


export function createDefaultStorage(): Storage | null {
  return typeof sessionStorage !== "undefined" ? sessionStorage : null;
}

@NgModule({
  declarations: [
    ...components,
  ],
  imports: [
    // angular
    BrowserModule,
    BrowserAnimationsModule,
    HttpClientModule,

    // NgXs
    NgxsModule.forRoot([
      AppState,
    ], {
      developmentMode: !environment.production, // Allow to enable freeze store in dev env
    }),

    NgxsModule.forRoot([
      AppState,
    ], {
      developmentMode: !environment.production, // Allow to enable freeze store in dev env
    }),
    NgxsLoggerPluginModule.forRoot({
      logger: console,
      collapsed: false,
      disabled: environment.production,
    }),
    NgxsReduxDevtoolsPluginModule.forRoot({
      disabled: environment.production,
    }),
    NgxsRouterPluginModule.forRoot(),

    // NgTranslate
    TranslateModule.forRoot({
      loader: {
        provide: TranslateLoader,
        useFactory: HttpLoaderFactory,
        deps: [HttpClient],
      },
    }),

    // Shared
    SharedModule,
    ApiModule,

    // App
    AppRoutingModule,
  ],
  providers: [
    {
      provide: ENVIRONMENT,
      useValue: environment,
    },
    {
      provide: APP_INITIALIZER,
      useFactory: appInitializerFn,
      multi: true,
      deps: [AppConfigService],
    },
    {
      provide:  HTTP_INTERCEPTORS,
      useClass: OAuth2Interceptor,
      multi: true,
    },
    {
      provide: OAuthStorage,
      useFactory: createDefaultStorage,
    },
    {
      provide: MatPaginatorIntl,
      useClass: CustomMatPaginatorIntlService,
    },
    {
      provide: ErrorHandler,
      useClass: GlobalErrorsHandlerService,
    },
    {
      provide: SNACK_BAR,
      useClass: MatSnackBar,
    },
  ],
  bootstrap: [AppComponent],
})
export class AppModule {
}

// required for AOT compilation
export function HttpLoaderFactory(http: HttpClient): TranslateHttpLoader {
  return new TranslateHttpLoader(http, "./assets/i18n/");
}
