import {ChangeDetectionStrategy, Component, Input} from "@angular/core";
import {FormBuilder, Validators} from "@angular/forms";
import {
  DepositsModel,
  LanguagesModel,
  LicensesModel,
  OrganizationalUnitsModel,
  PreservationPoliciesModel,
  SubmissionPoliciesModel,
} from "@app/generated-api";
import {SharedAbstractFormPresentational} from "@app/shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {BaseFormDefinition} from "@app/shared/models/base-form-definition.model";
import AccessModelEnum = DepositsModel.AccessModelEnum;
import {DateUtil, PropertyName} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-form",
  templateUrl: "./deposit-form.presentational.html",
  styleUrls: ["../../../../../shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositFormPresentational extends SharedAbstractFormPresentational<DepositsModel> {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  get accessModelEnum(): typeof AccessModelEnum {
    return AccessModelEnum;
  }

  @Input()
  languages: LanguagesModel[];

  @Input()
  licenses: LicensesModel[];

  @Input()
  submissionPolicies: SubmissionPoliciesModel[];

  @Input()
  preservationPolicies: PreservationPoliciesModel[];

  @Input()
  organizationalUnits: OrganizationalUnitsModel[];

  constructor(protected fb: FormBuilder) {
    super();
  }

  protected initNewForm(): void {
    this.form = this.fb.group({
      [this.formDefinition.organizationalUnitId]: ["", Validators.required],
      [this.formDefinition.title]: ["", Validators.required],
      [this.formDefinition.description]: ["", Validators.required],
      [this.formDefinition.languageId]: [""],
      [this.formDefinition.publicationDate]: [""],
      [this.formDefinition.collectionBegin]: [""],
      [this.formDefinition.collectionEnd]: [""],
      [this.formDefinition.author]: [""],
      [this.formDefinition.accessLevel]: [AccessModelEnum.PUBLIC],
      [this.formDefinition.hasEmbargo]: [false],
      [this.formDefinition.licenseId]: [""],
      [this.formDefinition.submissionPolicyId]: [""],
      [this.formDefinition.preservationPolicyId]: [""],
    });
  }

  protected bindFormTo(deposit: DepositsModel): void {
    this.form = this.fb.group({
      [this.formDefinition.organizationalUnitId]: [deposit.organizationalUnitId, Validators.required],
      [this.formDefinition.title]: [deposit.title, Validators.required],
      [this.formDefinition.description]: [deposit.description, Validators.required],
      [this.formDefinition.languageId]: [deposit.languageId],
      [this.formDefinition.publicationDate]: [deposit.publicationDate],
      [this.formDefinition.collectionBegin]: [DateUtil.convertOffsetDateTimeIso8601ToDate(deposit.collectionBegin)],
      [this.formDefinition.collectionEnd]: [DateUtil.convertOffsetDateTimeIso8601ToDate(deposit.collectionEnd)],
      [this.formDefinition.author]: [""],
      [this.formDefinition.accessLevel]: [deposit.access],
      [this.formDefinition.hasEmbargo]: [deposit.hasEmbargo],
      [this.formDefinition.licenseId]: [deposit.licenseId],
      [this.formDefinition.submissionPolicyId]: [deposit.submissionPolicyId],
      [this.formDefinition.preservationPolicyId]: [deposit.preservationPolicyId],
    });
  }

  protected treatmentBeforeSubmit(deposit: DepositsModel): DepositsModel {
    deposit.publicationDate = DateUtil.convertToLocalDateDateSimple(deposit.publicationDate);
    deposit.collectionBegin = DateUtil.convertToOffsetDateTimeIso8601(deposit.collectionBegin);
    deposit.collectionEnd = DateUtil.convertToOffsetDateTimeIso8601(deposit.collectionEnd);
    return deposit;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() organizationalUnitId: string;
  @PropertyName() title: string;
  @PropertyName() description: string;
  @PropertyName() languageId: string;
  @PropertyName() publicationDate: string;
  @PropertyName() collectionBegin: string;
  @PropertyName() collectionEnd: string;
  @PropertyName() author: string;
  @PropertyName() accessLevel: string;
  @PropertyName() hasEmbargo: string;
  @PropertyName() licenseId: string;
  @PropertyName() submissionPolicyId: string;
  @PropertyName() preservationPolicyId: string;
}
