import {ChangeDetectionStrategy, Component, OnInit} from "@angular/core";
import {MatDialog} from "@angular/material";
import {ActivatedRoute} from "@angular/router";
import {DepositDataFile} from "@app/features/deposit/models/deposit-data-file.model";
import {DepositDataFileAction} from "@app/features/deposit/stores/data-file/deposit-data-file.action";
import {depositActionNameSpace} from "@app/features/deposit/stores/deposit.action";
import {DepositStateModel} from "@app/features/deposit/stores/deposit.state";
import {DepositsModel, LanguagesModel, LicensesModel, PreservationPoliciesModel, SubmissionPoliciesModel} from "@app/generated-api";
import {OrganizationalUnitModule} from "@app/features/organizational-unit/organizational-unit.module";
import {LocalStateEnum} from "../../../../../shared/enums/localStateEnum";
import {LocalStateModel} from "@app/shared/models/localStateModel";
import {OrgUnitAction} from "@app/shared/stores/organizational-unit/organizational-unit.action";
import {SharedAbstractDetailRoutable} from "@app/shared/components/routables/shared-abstract-detail/shared-abstract-detail.routable";
import {Select, Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {TRANSLATE} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-detail-routable",
  templateUrl: "./deposit-detail.routable.html",
  styleUrls: ["../../../../../shared/components/routables/shared-abstract-detail/shared-abstract-detail.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositDetailRoutable extends SharedAbstractDetailRoutable<DepositsModel, DepositStateModel> implements OnInit {
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.language].list) languagesObs: Observable<LanguagesModel[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.license].list) licensesObs: Observable<LicensesModel[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.submissionPolicy].list) submissionPoliciesObs: Observable<SubmissionPoliciesModel[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.preservationPolicy].list) preservationPoliciesObs: Observable<PreservationPoliciesModel[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.organizationalUnit].list) organizationalUnitsObs: Observable<OrganizationalUnitModule[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.deposit].listDataFile) listDataFileObs: Observable<DepositDataFile[]>;

  readonly KEY_PARAM_NAME: string = "title";
  readonly KEY_DELETE_BUTTON: string = TRANSLATE("deposit.delete");
  readonly KEY_EDIT_BUTTON: string = TRANSLATE("deposit.edit");

  constructor(protected store: Store,
              protected route: ActivatedRoute,
              public dialog: MatDialog) {
    super(store, route, dialog, LocalStateEnum.deposit, depositActionNameSpace);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.store.dispatch(new OrgUnitAction.GetAll());
  }

  getSubResourceWithParentId(id: string): void {
    this.store.dispatch(new DepositDataFileAction.GetAll(id));
  }
}
