import {ChangeDetectionStrategy, Component, Input, OnChanges, SimpleChanges} from "@angular/core";
import {FormBuilder, Validators} from "@angular/forms";
import {OrganizationalUnitsModel, PreservationPoliciesModel, SubmissionPoliciesModel} from "@app/generated-api";
import {SharedAbstractFormPresentational} from "@app/shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {LocalModelAttributeEnum} from "@app/shared/enums/model-attribute.enum";
import {BaseFormDefinition} from "@app/shared/models/base-form-definition.model";
import _ from "lodash";
import {PropertyName, DateUtil} from "solidify-frontend";

@Component({
  selector: "dlcm-orgunit-form",
  templateUrl: "./orgunit-form.presentational.html",
  styleUrls: ["../../../../../shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class OrgunitFormPresentational extends SharedAbstractFormPresentational<OrganizationalUnitsModel> implements OnChanges {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  @Input()
  selectedSubmissionPolicies: SubmissionPoliciesModel[];

  @Input()
  listSubmissionPolicies: SubmissionPoliciesModel[];

  @Input()
  listPreservationPolicies: PreservationPoliciesModel[];

  constructor(private fb: FormBuilder) {
    super();
  }

  ngOnChanges(changes: SimpleChanges): void {
    super.ngOnChanges(changes);

    if (this.form === undefined) {
      return;
    }

    if (changes.selectedSubmissionPolicies && changes.selectedSubmissionPolicies.currentValue.length !== 0) {
      const newValue = _.map(changes.selectedSubmissionPolicies.currentValue, LocalModelAttributeEnum.resId);
      this.form.get(this.formDefinition.submissionPolicies).setValue(newValue);
    }
  }

  protected initNewForm(): void {
    this.form = this.fb.group({
      [this.formDefinition.name]: ["", Validators.required],
      [this.formDefinition.description]: ["", Validators.required],
      [this.formDefinition.openingDate]: [""],
      [this.formDefinition.closingDate]: [""],
      [this.formDefinition.isEmpty]: [false],
      [this.formDefinition.open]: [false],
      [this.formDefinition.submissionPolicies]: [],
    });
  }

  protected bindFormTo(organizationalUnit: OrganizationalUnitsModel): void {
    this.form = this.fb.group({
      [this.formDefinition.name]: [organizationalUnit.name, Validators.required],
      [this.formDefinition.description]: [organizationalUnit.description, Validators.required],
      [this.formDefinition.openingDate]: [organizationalUnit.openingDate],
      [this.formDefinition.closingDate]: [organizationalUnit.closingDate],
      [this.formDefinition.isEmpty]: [organizationalUnit.isEmpty],
      [this.formDefinition.open]: [organizationalUnit.open],
      [this.formDefinition.submissionPolicies]: [_.map(this.selectedSubmissionPolicies, LocalModelAttributeEnum.resId)],
    });
  }

  protected treatmentBeforeSubmit(organizationalUnit: OrganizationalUnitsModel): OrganizationalUnitsModel {
    organizationalUnit.openingDate = DateUtil.convertToLocalDateDateSimple(organizationalUnit.openingDate);
    organizationalUnit.closingDate = DateUtil.convertToLocalDateDateSimple(organizationalUnit.closingDate);
    return organizationalUnit;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() name: string;
  @PropertyName() description: string;
  @PropertyName() isEmpty: string;
  @PropertyName() closingDate: string;
  @PropertyName() openingDate: string;
  @PropertyName() submissionPolicies: string;
  @PropertyName() open: string;
}
