import {ChangeDetectionStrategy, Component} from "@angular/core";
import {MatDialog} from "@angular/material";
import {ActivatedRoute} from "@angular/router";
import {OrganizationalUnitsModel, PreservationPoliciesModel, SubmissionPoliciesModel} from "@app/generated-api";
import {LocalStateEnum} from "../../../../../shared/enums/localStateEnum";
import {LocalStateModel} from "@app/shared/models/localStateModel";
import {OrgUnitSubmissionPolicyAction} from "@app/shared/stores/organizational-unit/submission-policy/organizational-unit-submission-policy.action";
import {orgUnitActionNameSpace} from "@app/shared/stores/organizational-unit/organizational-unit.action";
import {OrganizationalUnitStateModel} from "@app/shared/stores/organizational-unit/organizational-unit.state";
import {SharedAbstractDetailRoutable} from "@app/shared/components/routables/shared-abstract-detail/shared-abstract-detail.routable";
import {Select, Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {TRANSLATE, StoreUtil} from "solidify-frontend";

@Component({
  selector: "dlcm-orgunit-detail-routable",
  templateUrl: "./orgunit-detail.presentational.html",
  styleUrls: ["../../../../../shared/components/routables/shared-abstract-detail/shared-abstract-detail.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class OrgunitDetailPresentational extends SharedAbstractDetailRoutable<OrganizationalUnitsModel, OrganizationalUnitStateModel> {
  @Select((state: LocalStateModel) => StoreUtil.isLoadingState(state[LocalStateEnum.organizationalUnit][LocalStateEnum.organizationalUnit_submissionPolicy])) isLoadingSubmissionPoliciesObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.organizationalUnit][LocalStateEnum.organizationalUnit_submissionPolicy].selected) selectedSubmissionPoliciesObs: Observable<SubmissionPoliciesModel[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.submissionPolicy].list) listSubmissionPoliciesObs: Observable<SubmissionPoliciesModel[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.preservationPolicy].list) listPreservationPoliciesObs: Observable<PreservationPoliciesModel[]>;

  readonly KEY_PARAM_NAME: string = "name";
  readonly KEY_DELETE_BUTTON: string = TRANSLATE("organizationalUnit.delete");
  readonly KEY_EDIT_BUTTON: string = TRANSLATE("organizationalUnit.edit");

  constructor(protected store: Store,
              protected route: ActivatedRoute,
              public dialog: MatDialog) {
    super(store, route, dialog, LocalStateEnum.organizationalUnit, orgUnitActionNameSpace);
  }

  getSubResourceWithParentId(id: string): void {
    this.store.dispatch(new OrgUnitSubmissionPolicyAction.GetAll(id));
  }
}
