import {ChangeDetectionStrategy, Component, OnInit} from "@angular/core";
import {ActivatedRoute} from "@angular/router";
import {OrganizationalUnitsModel, PreservationPoliciesModel, SubmissionPoliciesModel} from "@app/generated-api";
import {SharedAbstractEditRoutable} from "@app/shared/components/routables/shared-abstract-edit/shared-abstract-edit.routable";
import {LocalStateEnum} from "../../../../../shared/enums/localStateEnum";
import {LocalStateModel} from "@app/shared/models/localStateModel";
import {orgUnitActionNameSpace} from "@app/shared/stores/organizational-unit/organizational-unit.action";
import {OrganizationalUnitStateModel} from "@app/shared/stores/organizational-unit/organizational-unit.state";
import {OrgUnitSubmissionPolicyAction} from "@app/shared/stores/organizational-unit/submission-policy/organizational-unit-submission-policy.action";
import {Select, Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {StoreUtil} from "solidify-frontend";

@Component({
  selector: "dlcm-orgunit-edit-routable",
  templateUrl: "./orgunit-edit.presentational.html",
  styleUrls: ["../../../../../shared/components/routables/shared-abstract-edit/shared-abstract-edit.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class OrgunitEditPresentational extends SharedAbstractEditRoutable<OrganizationalUnitsModel, OrganizationalUnitStateModel> implements OnInit {
  @Select((state: LocalStateModel) => StoreUtil.isLoadingState(state[LocalStateEnum.organizationalUnit][LocalStateEnum.organizationalUnit_submissionPolicy])) isLoadingSubmissionPoliciesObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.organizationalUnit][LocalStateEnum.organizationalUnit_submissionPolicy].selected) selectedSubmissionPoliciesObs: Observable<SubmissionPoliciesModel[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.submissionPolicy].list) listSubmissionPoliciesObs: Observable<SubmissionPoliciesModel[]>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.shared][LocalStateEnum.preservationPolicy].list) listPreservationPoliciesObs: Observable<PreservationPoliciesModel[]>;

  constructor(protected store: Store,
              protected route: ActivatedRoute) {
    super(store, route, LocalStateEnum.organizationalUnit, orgUnitActionNameSpace);
  }

  getSubResourceWithParentId(id: string): void {
    this.store.dispatch(new OrgUnitSubmissionPolicyAction.GetAll(id));
  }

  ngOnInit(): void {
    super.ngOnInit();
  }
}
