import {ChangeDetectionStrategy, Component} from "@angular/core";
import {OrganizationalUnitsModel} from "@app/generated-api";
import {SharedAbstractListRoutable} from "@app/shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {orgUnitActionNameSpace} from "@app/shared/stores/organizational-unit/organizational-unit.action";
import {OrganizationalUnitStateModel} from "@app/shared/stores/organizational-unit/organizational-unit.state";
import {Store} from "@ngxs/store";
import {TRANSLATE} from "solidify-frontend";
import {LocalStateEnum} from "../../../../../shared/enums/localStateEnum";

@Component({
  selector: "dlcm-orgunit-list-routable",
  templateUrl: "../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class OrgunitListPresentational extends SharedAbstractListRoutable<OrganizationalUnitsModel, OrganizationalUnitStateModel> {
  readonly KEY_CREATE_BUTTON: string = TRANSLATE("organizationalUnit.new");
  readonly KEY_REFRESH_BUTTON: string = TRANSLATE("organizationalUnit.refresh");

  constructor(protected store: Store) {
    super(store, LocalStateEnum.organizationalUnit, orgUnitActionNameSpace);
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "name",
        header: TRANSLATE("organizationalUnit.table.header.name"),
        type: FieldTypeEnum.string,
        order: -1,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "description",
        header: TRANSLATE("organizationalUnit.table.header.description"),
        type: FieldTypeEnum.string,
        order: 0,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "creation.when" as any,
        header: TRANSLATE("organizationalUnit.table.header.creation.when"),
        type: FieldTypeEnum.datetime,
        order: 0,
        isFilterable: true,
        isSortable: true,
      },
    ];
  }
}
