import {ComponentType} from "@angular/cdk/portal";
import {OnInit} from "@angular/core";
import {MatDialog} from "@angular/material";
import {ActivatedRoute} from "@angular/router";
import {SharedAbstractDeleteDialog} from "@app/shared/components/dialogs/shared-abstract-delete/shared-abstract-delete.dialog";
import {SharedAbstractRoutable} from "@app/shared/components/routables/shared-abstract/shared-abstract.routable";
import {LocalModelAttributeEnum} from "@app/shared/enums/model-attribute.enum";
import {DeleteDialog} from "@app/shared/models/delete-dialog.model";
import {StoreDialogUtil} from "@app/shared/utils/store-dialog.util";
import {StoreRouteLocalUtil} from "@app/shared/utils/store-route-local.util";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {take} from "rxjs/operators";
import {ResourceActionHelper, ResourceNameSpace, ResourceStateModel, StoreUtil} from "solidify-frontend";
import {LocalStateEnum} from "../../../enums/localStateEnum";

export abstract class SharedAbstractDetailRoutable<T, U extends ResourceStateModel<T>> extends SharedAbstractRoutable implements OnInit {

  isLoadingObs: Observable<boolean>;
  currentObs: Observable<T>;

  abstract readonly KEY_PARAM_NAME: string;
  abstract readonly KEY_EDIT_BUTTON: string;
  abstract readonly KEY_DELETE_BUTTON: string;

  private resId: string;

  protected constructor(protected store: Store,
                        protected route: ActivatedRoute,
                        public dialog: MatDialog,
                        private state: LocalStateEnum,
                        private resourceNameSpace: ResourceNameSpace) {
    super();
    this.isLoadingObs = this.store.select(s => StoreUtil.isLoadingState(s[state] as U));
    this.currentObs = this.store.select(s => (s[state] as U).current);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.store.dispatch(ResourceActionHelper.loadResource(this.resourceNameSpace));
    this.retrieveCurrentModelWithUrl();
  }

  private retrieveCurrentModelWithUrl(): void {
    this.resId = this.route.snapshot.paramMap.get("id");
    this.getById(this.resId);
  }

  private getById(id: string): void {
    this.store.dispatch(ResourceActionHelper.getById(this.resourceNameSpace, id));
    this.getSubResourceWithParentId(id);
  }

  abstract getSubResourceWithParentId(id: string): void;

  getResId(): string {
    return this.resId;
  }

  edit(): void {
    this.store.dispatch(new Navigate([StoreRouteLocalUtil.getEditRoute(this.state), this.resId]));
  }

  delete(): void {
    const datas = {} as DeleteDialog;
    this.subscribe(
      this.currentObs.pipe(take(1)),
      (model: T) => {
        datas.name = model[this.KEY_PARAM_NAME];
        datas.resId = model[LocalModelAttributeEnum.resId];
      });
    const dialogRef = this.dialog.open(StoreDialogUtil.delete(this.state) as ComponentType<SharedAbstractDeleteDialog<T, U>>, {
      width: "250px",
      data: datas,
    });
  }
}
