import {CommonModule} from "@angular/common";
import {NgModule} from "@angular/core";
import {FormsModule, ReactiveFormsModule} from "@angular/forms";
import {MaterialModule} from "@app/material.module";
import {SharedDataTablePresentational} from "@app/shared/components/presentationals/shared-data-table/shared-data-table.presentational";
import {SharedSearchPresentational} from "@app/shared/components/presentationals/shared-search/shared-search.presentational";
import {SharedSearchableMultiSelectPresentational} from "@app/shared/components/presentationals/shared-searchable-multi-select/shared-searchable-multi-select.presentational";
import {SharedSearchableSingleSelectPresentational} from "@app/shared/components/presentationals/shared-searchable-single-select/shared-searchable-single-select.presentational";
import {LanguageState} from "@app/shared/stores/language/language.state";
import {LicenseState} from "@app/shared/stores/license/license.state";
import {OrganizationalUnitState} from "@app/shared/stores/organizational-unit/organizational-unit.state";
import {OrganizationalUnitSubmissionPolicyState} from "@app/shared/stores/organizational-unit/submission-policy/organizational-unit-submission-policy.state";
import {PreservationPolicyState} from "@app/shared/stores/preservation-policy/preservation-policy.state";
import {SharedState} from "@app/shared/stores/shared.state";
import {SubmissionPolicyState} from "@app/shared/stores/submission-policy/submission-policy.state";
import {FontAwesomeModule} from "@fortawesome/angular-fontawesome";
import {library} from "@fortawesome/fontawesome-svg-core";
import {fas} from "@fortawesome/free-solid-svg-icons";
import {TranslateModule} from "@ngx-translate/core";
import {NgxsModule} from "@ngxs/store";
import {TableModule} from "primeng/table";
import {SolidifyFrontendModule} from "solidify-frontend";
import {SharedBreadcrumbPresentational} from "./components/presentationals/shared-breadcrumb/shared-breadcrumb.presentational";
import {SharedPaginatorPresentational} from "./components/presentationals/shared-paginator/shared-paginator.presentational";
import {SharedSearchScopePresentational} from "./components/presentationals/shared-search-scope/shared-search-scope.presentational";

const views = [];
const dialogs = [];
const components = [
  SharedSearchPresentational,
  SharedDataTablePresentational,
  SharedPaginatorPresentational,
  SharedBreadcrumbPresentational,
  SharedSearchableSingleSelectPresentational,
  SharedSearchableMultiSelectPresentational,
  SharedSearchScopePresentational,
];
const directives = [];
const modules = [
  CommonModule,
  FormsModule,
  ReactiveFormsModule,
  MaterialModule,
  FontAwesomeModule,
  TableModule,
  TranslateModule,
  SolidifyFrontendModule,
];

@NgModule({
  declarations: [
    ...views,
    ...dialogs,
    ...components,
    ...directives,
  ],
  imports: [
    ...modules,
    NgxsModule.forFeature([
      SharedState,
      LicenseState,
      LanguageState,
      PreservationPolicyState,
      SubmissionPolicyState,
      OrganizationalUnitState,
      OrganizationalUnitSubmissionPolicyState,
    ]),
  ],
  exports: [
    ...modules,
    ...views,
    ...dialogs,
    ...components,
    ...directives,
  ],
})
export class SharedModule {
}

// Fontawesome library
library.add(fas);
