import {LocalStateEnum} from "../enums/localStateEnum";
import {LanguageAction} from "@app/shared/stores/language/language.action";
import {LanguageState, LanguageStateModel} from "@app/shared/stores/language/language.state";
import {LicenseAction} from "@app/shared/stores/license/license.action";
import {LicenseState, LicenseStateModel} from "@app/shared/stores/license/license.state";
import {PreservationPolicyAction} from "@app/shared/stores/preservation-policy/preservation-policy.action";
import {PreservationPolicyState, PreservationPolicyStateModel} from "@app/shared/stores/preservation-policy/preservation-policy.state";
import {SharedAction} from "@app/shared/stores/shared.action";
import {SubmissionPolicyAction} from "@app/shared/stores/submission-policy/submission-policy.action";
import {SubmissionPolicyState, SubmissionPolicyStateModel} from "@app/shared/stores/submission-policy/submission-policy.state";
import {Action, State, StateContext} from "@ngxs/store";
import {BaseState} from "solidify-frontend";

export interface SharedStateModel extends BaseState {
  [LocalStateEnum.language]: LanguageStateModel;
  [LocalStateEnum.license]: LicenseStateModel;
  [LocalStateEnum.submissionPolicy]: SubmissionPolicyStateModel;
  [LocalStateEnum.preservationPolicy]: PreservationPolicyStateModel;
}

@State<SharedStateModel>({
  name: LocalStateEnum.shared,
  defaults: {
    isLoadingCounter: 0,
    language: null,
    license: null,
    preservationPolicy: null,
    submissionPolicy: null,
  },
  children: [
    LanguageState,
    LicenseState,
    SubmissionPolicyState,
    PreservationPolicyState,
  ],
})
export class SharedState {

  @Action(SharedAction.GetAllResources)
  getAllResources(ctx: StateContext<SharedStateModel>, action: SharedAction.GetAllResources): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    ctx.dispatch([
      new LanguageAction.GetAll(),
      new LicenseAction.GetAll(),
      new PreservationPolicyAction.GetAll(),
      new SubmissionPolicyAction.GetAll(),
      new SharedAction.GetAllResourcesSuccess(), // TO FIX Should be raised only when GetAllLanguagesSuccess and GetAllLicensesSuccess is raised)
    ]);
  }

  @Action(SharedAction.GetAllResourcesSuccess)
  getAllResourcesSuccess(ctx: StateContext<SharedStateModel>, action: SharedAction.GetAllResourcesSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(SharedAction.GetAllResourcesFail)
  getAllResourcesFail(ctx: StateContext<SharedStateModel>): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }
}



